##############################
# Unmutate
##############################
# Author: T. Nelson Downs
##############################
# Version: 0.7
##############################

import pygame
import random
import data

pygame.init()
pygame.mixer.init()
pygame.mixer.music.load(data.filepath("deedee.ogg"))
pygame.mixer.music.play(-1)
SCREEN_W = 640
SCREEN_H = 480
screen = pygame.display.set_mode((SCREEN_W, SCREEN_H))
gamefont = pygame.font.SysFont("None", 24)

class Sound:
	jump = pygame.mixer.Sound(data.filepath("jump.ogg"))
	skid = pygame.mixer.Sound(data.filepath("skid.ogg"))
	born = pygame.mixer.Sound(data.filepath("born.ogg"))
	die = pygame.mixer.Sound(data.filepath("die.ogg"))

class Gender: # Written for codominance (In real life it's not codominance, but it makes the formulas all pretty :D)
	x = 1
	y = 4
	male = 5
	female = 2
	compatible = male + female


def rectcollidewithblock(rect, blocks):
	c1 = rect.x / Block.size
	r1 = rect.y / Block.size
	c2 = rect.right / Block.size
	r2 = rect.bottom / Block.size
	returnable = None
	for r in xrange(r1, r2 + 1):
		for c in xrange(c1, c2 + 1):
			if r < len(blocks) and r >= 0 and c < len(blocks[r]) and c >= 0:
				if blocks[r][c] != Block.none:
					if blocks[r][c] == Block.water:
						returnable = (c, r, blocks[r][c])
					elif blocks[r][c] == Block.grass:
						return (c, r, blocks[r][c])
	return returnable

class Block:
	none =  0
	grass = 1
	water = 2
	size = 32 

class Level:
	def __init__(self, filename, beings):
		self.x = 0
		self.y = 0
		self.h = 0
		self.w = 0
		self.filename = filename
		self.blocks = [] 
		x = 0
		y = 0
		for line in open(filename):
			self.blocks.append([])
			for c in line:
				if c == '#':
					self.blocks[y].append(Block.grass)	
				elif c == '~':
					self.blocks[y].append(Block.water)
				elif c.isdigit():
					beings[int(c)].rect.x = x * Block.size
					beings[int(c)].rect.y = y * Block.size
					self.blocks[y].append(Block.none)
				else:
					self.blocks[y].append(Block.none)
				x += 1
			if y > self.h:
				self.h = y
			if x - 1 > self.w:
				self.w = x
			y += 1
			x = 0
		self.w -= 1
		self.w *= Block.size
		self.h += 1
		self.h *= Block.size
	def follow(self, rect):
		self.x = rect.x - (rect.w + SCREEN_W) / 2 
		self.y = rect.y - (rect.h + SCREEN_H) / 2
	def constrain(self):
		maxx = self.w - SCREEN_W
		maxy = self.h - SCREEN_H
		if self.x < 0:
			self.x = 0
		elif self.x > maxx:
			self.x = maxx
		if self.y < 0:
			self.y = 0
		elif self.y > maxy:
			self.y = maxy

	def draw(self, surface):
		startx = max(self.x / Block.size, 0)
		endx = max((self.x + SCREEN_W) / Block.size + 1, 0)
		starty = max(self.y / Block.size, 0)
		endy = max((self.y + SCREEN_H) / Block.size + 1, 0)
		rows = len(self.blocks)
		if endy > rows:
			endy = rows 
		color_grass = ((0, 0x99, 0), (0, 0x77, 0))
		color_water = ((0x44, 0x77, 0xff), (0x33, 0x66, 0xff))
		for y in xrange(starty, endy):
			col = len(self.blocks[y])
			for x in xrange(startx, min(endx, col)):
				block = self.blocks[y][x]
				ci = (x + y) % 2
				if block == Block.grass:
					pygame.draw.rect(surface, color_grass[ci], (x * Block.size - self.x, y * Block.size - self.y, Block.size, Block.size))
				elif block == Block.water:
					pygame.draw.rect(surface, color_water[ci], (x * Block.size - self.x, y * Block.size - self.y, Block.size, Block.size))
				elif (x + y) % 8 == 0:
					pygame.draw.rect(surface, (0x44, 0xff, 0xff), (x * Block.size - self.x, y * Block.size - self.y, Block.size, Block.size))
	def check_keys(self, keys):
		speed = 10
		if keys[pygame.K_UP] or keys[pygame.K_w]:
			self.y -= speed 
		if keys[pygame.K_DOWN] or keys[pygame.K_s]:
			self.y += speed 
		if keys[pygame.K_RIGHT] or keys[pygame.K_d]:
			self.x += speed 
		if keys[pygame.K_LEFT] or keys[pygame.K_a]:
			self.x -= speed

class Stat:
	dead = 0
	normal = 1
	frozen = -1

class Gen: # Genotype
	# Types of dominance
	men = 0 # Mendelian
	co = 1 # Codominant
	# Dominant?
	do = 0
	rec = 1
	t = 3 # This is the index for the type of dominance
	mut = 2 
	chros = (
			(0.75, 1.5, 0, co),
			(10, 50, 0, co),
			(.25, 0.5, 0, co),
			(.25, 1, 0, co),
			(4, 9, 0, co),
			(0.1, 0.5, 0, co),
			(Gender.y, Gender.x, 0, co),
			(2, 3, 0, co),
			(0.5, 1, 0, men),
			(0.5, 1, 0, men),
			(True, True, 0, men))
	@staticmethod
	def toval(i, chro):
		if Gen.chros[i][Gen.t] == Gen.men:
			return Gen.chros[i][min(chro[0], chro[1])]
		elif Gen.chros[i][Gen.t] == Gen.co:
			return Gen.chros[i][chro[0]] + Gen.chros[i][chro[1]]	
	@staticmethod
	def split(chro):
		if not random.randint(0, 20):
			return Gen.mut
		return random.choice(chro)

class Phe: # Phenotype enum
	notation = (('B', 'L'), ('H', 'W'), ('F', 'S'), ('T', 'V'), ('D', 'E'), ('G', 'M'), ('Y', 'X'), ('C', 'R'), ('Z', 'z'), ('W', 'w'), ('I', 'I'))
	size = 0
	health = 1
	agility = 2
	leg_length = 3
	jump = 4
	foot_size = 5
	gender = 6
	shape = 7
	green = 8
	wing_size = 9
	invulnerable = 10
	@staticmethod
	def create(genotype):
		attr = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
		for i in xrange(len(genotype)):
			attr[i] = Gen.toval(i, genotype[i])
		attr[Phe.size] *= 16
		if attr[Phe.leg_length] > 0:
			attr[Phe.agility] *= attr[Phe.leg_length] 
		elif attr[Phe.shape] == 4 or attr[Phe.shape] == 5:
			attr[Phe.agility] *= .1
		attr[Phe.leg_length] = int(attr[Phe.leg_length] * attr[Phe.size])
		attr[Phe.foot_size] *= attr[Phe.size]
		attr[Phe.wing_size] *= attr[Phe.size]
		return attr
	@staticmethod
	def breed(mom, dad):
		genotype = []
		if Gen.toval(Phe.gender, mom[Phe.gender]) + Gen.toval(Phe.gender, dad[Phe.gender]) != Gender.compatible:
			return None
		for i in xrange(len(mom)):
			genotype.append([Gen.split(mom[i]), Gen.split(dad[i])])
		return genotype

class Being:
	def __init__(self, x, y, genotype):
		self.genotype = genotype
		self.attr = Phe.create(genotype)
		self.rect = pygame.Rect(x, y, self.attr[Phe.size], self.attr[Phe.size] + self.attr[Phe.leg_length])
		self.color = [0, self.attr[Phe.green] * 200, 0]
		if self.attr[Phe.gender] == Gender.female:
			self.color[0] = 200
		elif self.attr[Phe.gender] == Gender.male:
			self.color[2] = 200
		self.mutation = 0
		self.health = self.attr[Phe.health]
		self.status = Stat.normal
		self.bounce = 0.9 
		if self.attr[Phe.shape] != 4 and self.attr[Phe.shape] != 5 and self.attr[Phe.leg_length] <= 0:
			self.bounce = -0.9
		if self.attr[Phe.leg_length] <= 0:
			self.attr[Phe.jump] = 0
		for chro in genotype:
			if chro[0] == Gen.mut:
				self.mutation += 50 
			if chro[1] == Gen.mut:
				self.mutation += 50 
		if self.mutation > 255:
			self.mutation = 255
		if self.rect.w <= 0:
			self.status = Stat.dead
		# Velocity
		self.vx = 0
		self.vy = 0 
		#self.friction = 1 - (self.attr[Phe.foot_size] / self.attr[Phe.size]) * 0.1
		self.ONG = True # On ground
	def draw(self, surface, level):
		newx = self.rect.x - level.x
		newy = self.rect.y - level.y

		if self.status == Stat.frozen:
			pygame.draw.rect(surface, 0xeeeeff, (newx - 20, newy - 2, self.rect.w + 40, self.rect.h + 2))

		# Wings
		if self.attr[Phe.wing_size] > 0:
			wstep = random.randint(0, 7) * (self.rect.w / 6)
			pygame.draw.polygon(surface, 0xffffff, (
					(newx + self.rect.w / 4, newy + self.rect.w / 2),
					(newx - self.attr[Phe.wing_size], newy + wstep),
					(newx + self.rect.w / 4, newy + self.rect.w)))			
			pygame.draw.polygon(surface, 0xffffff, (
					(newx + self.rect.w * 3 / 4, newy + self.rect.w / 2),
					(newx + + self.rect.w + self.attr[Phe.wing_size], newy + wstep),
					(newx + self.rect.w * 3 / 4, newy + self.rect.w)))			
		# Body
		if self.health <= 10 and not random.randint(0, 3) and not self.attr[Phe.invulnerable]:
			color = (random.randint(100, 255), 100, 100)
		else:
			color = self.color 
		dampen = .7
		darkcolor = (color[0] * dampen, color[1] * dampen, color[2] * dampen)
		if self.attr[Phe.shape] == 4:
			pygame.draw.rect(surface, color, (newx, newy, self.rect.w, self.attr[Phe.size]))
		elif self.attr[Phe.shape] == 5:
			shape = ((newx + self.rect.w / 2, newy),
					(newx + self.rect.w, newy + self.rect.w / 2),
					(newx + self.rect.w, newy + self.rect.w),
					(newx, newy + self.rect.w),
					(newx, newy + self.rect.w / 2))
			pygame.draw.polygon(surface, color, shape)
		else:
			pygame.draw.circle(surface, color, (newx + self.rect.w / 2, newy + self.rect.w / 2), self.rect.w / 2)
			if self.attr[Phe.leg_length] > 0:
				pygame.draw.polygon(surface, color, ((newx + self.rect.w / 2, newy + self.rect.w / 2), (newx, newy + self.rect.w), (newx + self.rect.w, newy + self.rect.w)))
			
		# Legs
		if self.attr[Phe.leg_length] > 0:
			step = (((self.rect.x + self.rect.y / 4) % self.attr[Phe.leg_length]) if ((self.rect.y / 4 + self.rect.x) / self.attr[Phe.leg_length] % 2) else (self.attr[Phe.leg_length] - (self.rect.x + self.rect.y / 4) % self.attr[Phe.leg_length]))
			legF = ((newx + self.rect.w / 2, newy + self.rect.w),
					(newx + self.rect.w, newy + self.rect.w),
					(newx + step + self.attr[Phe.foot_size], newy + self.rect.h),
					(newx + step, newy + self.rect.h))
			legB = ((newx, newy + self.rect.w),
					(newx + self.rect.w / 2, newy + self.rect.w),
					(newx - step + self.rect.w, newy + self.rect.h),
					(newx - step + self.rect.w - self.attr[Phe.foot_size], newy + self.rect.h))
			pygame.draw.polygon(surface, color, legF)
			pygame.draw.polygon(surface, color, legB)
		# Eyes
		pygame.draw.circle(surface, darkcolor, (newx + self.rect.w / 2, newy + self.rect.w / 2), self.rect.w / 3)
		pygame.draw.circle(surface, 0xFFFFFF - 0x010001 * self.mutation, (newx + self.rect.w / 2, newy + self.rect.w / 2), self.rect.w / 4)
		eye_offset = [self.rect.w / 6, self.rect.w / 6]
		if self.vx <= -1:
			eye_offset[0] *= -1
		elif self.vx < 1:
			eye_offset[0] = 0
		if self.vy <= -1:
			eye_offset[1] *= -1
		elif self.vy < 1:
			eye_offset[1] = 0
		pygame.draw.circle(surface, 0, (newx + self.rect.w / 2 + eye_offset[0], newy + self.rect.w / 2 + eye_offset[1]), self.rect.w / 6)
		# Health
		if not self.attr[Phe.invulnerable]:
			pygame.draw.rect(surface, (0xBB, 0, 0), (newx + (-self.attr[Phe.health] + self.rect.w) / 2, newy - 20, self.health, 10))
			pygame.draw.rect(surface, (0, 0xFF, 0), (newx + (-self.attr[Phe.health] + self.rect.w) / 2, newy - 20, self.attr[Phe.health], 10), 2)
		# Ice lines
		if self.status == Stat.frozen:
			for i in xrange(5, self.rect.h, 3):
				pygame.draw.line(surface, 0xaaccff, (newx - 20, newy + i), (newx + 20 + self.rect.w, newy + i - 5))
	def draw_focus(self, surface, level):
		pygame.draw.rect(surface, 0xffff00 + random.randint(0x00, 0x99), (self.rect.x - self.attr[Phe.leg_length] / 2 - level.x, self.rect.y - 5 - level.y, self.rect.w + self.attr[Phe.leg_length], self.rect.h + 10), 2)
	def draw_genome(self, surface):
		pygame.draw.rect(surface, 0, (0, 0, surface.get_width(), 30))
		x = 10
		y = 10
		for i in xrange(len(self.genotype)):
			chro = self.genotype[i]
			if chro[0] == Gen.mut:
				a = '#'
			else:
				a = Phe.notation[i][chro[0]]
			if chro[1] == Gen.mut:
				b = '#'
			else:
				b = Phe.notation[i][chro[1]]
			if chro[0] < chro[1]:
				order = a + b
			else:
				order = b + a
			surface.blit(gamefont.render(order, 1, (0xFF, 0xFF, 0)), (x, y))
			x += 40 
	def check_keys(self, keys):
		if self.status != Stat.frozen:
			if (keys[pygame.K_UP] or keys[pygame.K_w]):
				if self.ONG:
					self.vy = - self.attr[Phe.jump]
					if self.attr[Phe.jump] > 0:
						Sound.jump.play()
				elif self.attr[Phe.wing_size] > 0:
					if self.vy > 1:
						self.vy *= 0
			if keys[pygame.K_LEFT] or keys[pygame.K_a]:
				self.vx -= self.attr[Phe.agility]
				if not pygame.mixer.get_busy():
					Sound.skid.play()
			if keys[pygame.K_RIGHT] or keys[pygame.K_d]:
				self.vx += self.attr[Phe.agility]
				if not pygame.mixer.get_busy():
					Sound.skid.play()
			if keys[pygame.K_DOWN] or keys[pygame.K_s]:
				if self.bounce >= 0:
					self.vy += 3 
				else:
					self.vy += .7
	def update(self, blocks):
		self.ONG = False
		self.vy += .5
		self.vx *= 0.9

		# X collisions
		crect = self.rect
		self.rect = self.rect.move(self.vx, 0)
		crect = crect.union(self.rect)
		collision = rectcollidewithblock(crect, blocks)
		if collision != None:
			if collision[2] == Block.grass:
				if self.vx >= 1:
					self.rect.right = collision[0] * Block.size - 1
					self.vx *= self.bounce
				else:
					self.rect.x = (collision[0] + 1) * Block.size + 1
					self.vx *= self.bounce

		# Y collisions
		crect = self.rect
		self.rect = self.rect.move(0, self.vy)
		crect = crect.union(self.rect)
		collision = rectcollidewithblock(crect, blocks)
		if collision != None:
			if collision[2] == Block.grass:
				if self.vy >= 1:
					self.rect.bottom = collision[1] * Block.size - 1
					if self.bounce < 0:
						self.vy *= self.bounce 
						if self.vy > -7:
							self.vy = -7
					else:
							self.vy = 0
					self.ONG = True
				else:
					self.rect.y = (collision[1] + 1) * Block.size + 1
					self.vy *= self.bounce 
			elif collision[2] == Block.water:
				if self.vy >= 1:
					self.vy *= (1 - self.attr[Phe.foot_size] / self.attr[Phe.size])
					if self.attr[Phe.foot_size] / self.attr[Phe.size] == 1:
						self.ONG = True
						self.vy = 0

		# Manage Health
		if not self.attr[Phe.invulnerable]:
			if self.status != Stat.frozen:
				self.health -= .005
				if self.health <= 1:
					self.status = Stat.dead
	def __str__(self):
		chros = str() 
		for i in self.genotype:
			chros += str(i)	
		return chros + " (x: " + str(self.rect.x) + " y: " + str(self.rect.y) + ")"

def main():
	pygame.display.set_caption("Unmutate")
	clock = pygame.time.Clock()

	text = ["Welcome to Unmutate.",
			"Press 's' if you would like to skip these messages.",
			"Guide the mendelians to immortality.",
			"They once where immortal,",
			"but those genes were mutated",
			" and now the mendelians die after too much time passes.",
			"Click on a mendelian to select it.",
			"Cycle through the creatures with 'tab'.",
			"Press 'b' to breed two opposite sex mendelians while they are overlapping.",
			"Press 'f' to have a mendelian freeze another.",
			"Press 'm' to have a mendelian unfreeze another.",
			"Press 'k' to kill a mendelian.",
			"The selected mendelian's genotype appears at the top of the screen.",
			"Mutations are depicted as '#' in the genotype and set a gene to 0.",
			"The other symbols at the top stand for different genes.",
			"The pairs that have two different letters are codominant traits,",
			"and the pairs with two of the same letter but different case",
			"are mendelian traits.",
			"E.g. 'Aa' would show the trait 'A' because it is dominant over 'a'.",
			"However, 'aa' would show the trait 'a' because there isn't a",
			"dominant trait to overpower it.",
			"Something like 'WH' would show a mixture of 'W' and 'H'.",
			"Something like 'WW' or 'HH' would show 'W' or 'H' respectively.",
			"If you didn't get all that can press 'r' to restart these messages."]
	texti = 0

	# Intro:
	while texti < len(text):
		clock.tick(30)	
		screen.fill(0)
		screen.blit(gamefont.render(text[texti], 0, (0xff, 0xff, 0xff)), (20, 100))
		pygame.display.flip()
		for event in pygame.event.get():
			if event.type == pygame.KEYDOWN:
				if event.key == pygame.K_SPACE or event.key == pygame.K_RETURN:
					texti += 1
				elif event.key == pygame.K_r:
					texti = 0
				elif event.key == pygame.K_s:
					texti = len(text)
			if event.type == pygame.QUIT:
				return

	genotype_a = [
			(Gen.do, Gen.rec),
			(Gen.do, Gen.rec),
			(Gen.do, Gen.rec),
			(Gen.do, Gen.rec),
			(Gen.do, Gen.rec),
			(Gen.do, Gen.rec),
			(Gen.do, Gen.rec),
			(Gen.do, Gen.rec),
			(Gen.do, Gen.rec),
			(Gen.mut, Gen.mut),
			(Gen.mut, Gen.mut)]
	
	genotype_c = [
	(Gen.do, Gen.rec),
	(Gen.do, Gen.do),
	(Gen.do, Gen.rec),
	(Gen.do, Gen.do),
	(Gen.do, Gen.do),
	(Gen.rec, Gen.rec),
	(Gen.rec, Gen.rec),
	(Gen.do, Gen.rec),
	(Gen.do, Gen.rec),
	(Gen.do, Gen.rec),
	(Gen.mut, Gen.mut)]
	
	genotype_d = [
	(Gen.do, Gen.do),
	(Gen.do, Gen.do),
	(Gen.do, Gen.rec),
	(Gen.do, Gen.do),
	(Gen.do, Gen.rec),
	(Gen.rec, Gen.rec),
	(Gen.do, Gen.rec),
	(Gen.do, Gen.rec),
	(Gen.mut, Gen.mut),
	(Gen.mut, Gen.mut),
	(Gen.do, Gen.do)]
	
	genotype_b = list(genotype_a)
	genotype_b[Phe.gender] = (Gen.rec, Gen.rec)

	beings = [
			Being(200, 200, genotype_a),
			Being(300, 200, genotype_b),
			Being(0, 0, genotype_c),
			Being(0, 0, genotype_d)]
	current = beings[0]

	goal = beings[3]
	
	genotype_b = list(genotype_a)
	genotype_b[Phe.gender] = (Gen.rec, Gen.rec)

	beings[2].status = Stat.frozen
	beings[3].status = Stat.frozen

	level = Level(data.filepath("world.txt"), beings)

	random.seed()
	while 1:
		x = 0
		y = 0
		clock.tick(60)
		for event in pygame.event.get():
			if event.type == pygame.QUIT or (event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE):
				return
			elif event.type == pygame.KEYDOWN:
				if event.key == pygame.K_k and current != None:
					current.status = Stat.dead
				elif event.key == pygame.K_f:
					if current != None:
						for being in beings:
							if being.rect.colliderect(current.rect) and being != current:
								being.status = Stat.frozen
				elif event.key == pygame.K_m:
					if current != None:
						for being in beings:
							if being.rect.colliderect(current.rect) and being != current:
								being.status = Stat.normal
				elif event.key == pygame.K_TAB:
					done = False
					for a in xrange(len(beings)):
						if beings[a] == current and not done:
							for b in xrange(a + 1, a + len(beings)):
								if not done:
									temp = beings[b % len(beings)]
									if temp.status == Stat.normal:
										current = temp
										done = True
				elif event.key == pygame.K_b:
					if current != None:
						for being in beings:
							if being.attr[Phe.gender] + current.attr[Phe.gender] == Gender.compatible and being.rect.colliderect(current) and being.status == Stat.normal and current.status == Stat.normal:
								beings.append(Being((being.rect.x + current.rect.x) / 2, (being.rect.y + current.rect.y) / 2, Phe.breed(being.genotype, current.genotype)))
								Sound.born.play()
								current.vy -= 6
								being.vy -= 3
								if being.rect.x < current.rect.x:
									being.vx -= 10
									current.vx += 10 
								else:
									being.vx += 10
									current.vx -= 10 
								#being.health -= 10
								#current.health -= 10
								beings[-1].vy -= 8
								beings[-1].rect.bottom = current.rect.bottom
								beings[-1].vx += 2
								beings[-1].rect.bottom = (current.rect.bottom + being.rect.bottom) / 2
								break
			if event.type == pygame.MOUSEBUTTONDOWN:
				x, y = pygame.mouse.get_pos()
				x += level.x
				y += level.y
				for being in beings:
					if being.rect.collidepoint((x, y)):
						current = being

		keys = pygame.key.get_pressed()

		dead = None
		for i in xrange(len(beings)):
			if beings[i].status == Stat.dead:
				dead = i
			else:
				beings[i].update(level.blocks)
		if dead != None:
			Sound.die.play()
			if beings[dead] == current:
				current = beings[0]
			del beings[dead]

		if current.status != Stat.normal:
			for being in beings:
				if being.status == Stat.normal:
					current = being
		
		current.check_keys(keys)
		level.follow(current.rect)
		level.constrain()

		for a in xrange(0, len(beings)):
			for b in xrange(a + 1, len(beings)):
				if beings[a].rect.colliderect(beings[b].rect):
					if beings[a].rect.x < beings[b].rect.x:
						if beings[a].status == Stat.normal:
							beings[a].vx -= .2
						if beings[b].status == Stat.normal:
							beings[b].vx += .2
					else:	
						if beings[a].status == Stat.normal:
							beings[a].vx += .2
						if beings[b].status == Stat.normal:
							beings[b].vx -= .2
			if beings[a].rect.y > level.h:
				beings[a].status = Stat.dead

		if goal != None and goal.status == Stat.normal:
			text = ["You found a mendelian with immortality!",
					"You've beaten the game. But you can still keep playing :)"]
			texti = 0
			while texti < len(text):
				screen.fill(0)
				screen.blit(gamefont.render(text[texti], 1, (255, 255, 0)), (10, 10))
				pygame.display.flip()
				for event in pygame.event.get():
					if event.type == pygame.KEYDOWN:
						if event.key == pygame.K_RETURN or event.key == pygame.K_SPACE:
							texti += 1
			goal = None

		# Draw the screen
		screen.fill(0x88FFFF)
		level.draw(screen)
		for being in beings:
			being.draw(screen, level)
		if current != None:
			current.draw_focus(screen, level)
			current.draw_genome(screen)
		pygame.display.update()
